package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderLog;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class MesWorkOrderService extends BaseMesService<MesWorkOrder> implements IMesWorkOrderService {
    @Autowired
    private MesWorkOrderLogService workOrderLogService;

    @Override
    public MesWorkOrder insert(MesWorkOrder bean) {
        //插入前构造数据
        onInsertBean(bean);

        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            List<MesWorkOrder> sortWorkOrderList = new ArrayList<>();
            List<MesWorkOrderLog> orderLogList = new ArrayList<>();
            for (int i = 0; i < bean.getSortInsertOrderQty(); i++) {
                MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
                BeanUtils.copyProperties(bean, copyMesWorkOrder);
                copyMesWorkOrder.setQty(1D);
                ConvertBean.serviceModelInitialize(copyMesWorkOrder, AuthUtil.getSessionUser().getUserName());
                sortWorkOrderList.add(copyMesWorkOrder);

                MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
                BeanUtils.copyProperties(copyMesWorkOrder, workOrderLog, "id");
                orderLogList.add(workOrderLog);

            }
            insertBatch(sortWorkOrderList);
            workOrderLogService.insertBatch(orderLogList);
        }

        bean.setQty(bean.getSortInsertOrderQty().doubleValue());
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(bean, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public MesWorkOrder update(MesWorkOrder bean) {
        //插入前构造数据
        onInsertBean(bean);

        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("工单类型为【%s】 不需要修改数量", MesExtEnumUtil.ORDER_TYPE.SORT.getValue()))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }

        bean.setQty(bean.getSortInsertOrderQty().doubleValue());
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(bean, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);

        bean = baseRDao.insert(bean);
        return bean;
    }


    protected void setPackQueryBean(MesWorkOrder bean, DdlPackBean packBean) {
    }

    protected void onInsertBean(MesWorkOrder item) {

    }

    protected void onUpdateBean(MesWorkOrder item) {

    }

}
